@@ -141,7 +141,7 @@ Menu.prototype = {
   open : function(contextElement) {
     // We have to open the context menu
     var menu = this._menu.getNode();
-    if (menu.localName == "menupopup" &&
+    if ((menu.localName == "popup" || menu.localName == "menupopup") &&
         contextElement && contextElement.exists()) {
       this._controller.rightClick(contextElement);
       this._controller.waitFor(function() {
@@ -589,6 +589,11 @@ MozMillController.prototype.check = function(el, state) {
     return false;
   }
 
+  // If we have a XUL element, unwrap its XPCNativeWrapper
+  if (element.namespaceURI == "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul") {
+    element = utils.unwrapNode(element);
+  }
+  
   state = (typeof(state) == "boolean") ? state : false;
   if (state != element.checked) {
     this.click(el);
@@ -616,7 +621,12 @@ MozMillController.prototype.radio = function(el)
 
   this.click(el);
   this.waitFor(function() {
-    return element.selected == true;
+    // If we have a XUL element, unwrap its XPCNativeWrapper
+    if (element.namespaceURI == "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul") {
+  	  element = utils.unwrapNode(element);
+  	  return element.selected == true;
+    }
+    return element.checked == true;
   }, "Radio button " + el.getInfo() + " could not be selected", 500);
 
   frame.events.pass({'function':'Controller.radio(' + el.getInfo() + ')'});
@@ -646,6 +656,14 @@ MozMillController.prototype.waitForElement = function(elem, timeout, interval) {
   frame.events.pass({'function':'Controller.waitForElement()'});
 }
 
+MozMillController.prototype.waitForElementNotPresent = function(elem, timeout, interval) {
+  this.waitFor(function() {
+    return !elem.exists();
+  }, "Timeout exceeded for waitForElementNotPresent " + elem.getInfo(), timeout, interval);
+
+  frame.events.pass({'function':'Controller.waitForElementNotPresent()'});
+}
+
 MozMillController.prototype.__defineGetter__("waitForEvents", function() {
   if (this._waitForEvents == undefined)
     this._waitForEvents = new waitForEvents();
@@ -749,13 +767,19 @@ MozMillController.prototype.select = function (el, indx, option, value) {
    }
   }
   //if we have a xul menulist select accordingly
-  else if (element.localName.toLowerCase() == "menulist"){
+  else if (element.localName.toLowerCase() == "menulist") {
+    ownerDoc = element.ownerDocument;
+    // Unwrap the XUL element's XPCNativeWrapper
+    if (element.namespaceURI.toLowerCase() == "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul") {
+      element = utils.unwrapNode(element);
+    }
+    
     var item = null;
 
     if (indx != undefined) {
       if (indx == -1) {
         events.triggerEvent(element, 'focus', false);
-      element.selectedIndex = indx;
+        element.selectedIndex = indx;
         events.triggerEvent(element, 'change', true);
 
         frame.events.pass({'function':'Controller.select()'});
@@ -770,30 +794,30 @@ MozMillController.prototype.select = function (el, indx, option, value) {
             value != undefined && entry.value == value) {
           item = entry;
           break;
+        }
+      }
     }
-    }
-  }
 
     // Click the item
     try {
-      EventUtils.synthesizeMouse(element, 1, 1, {}, item.ownerDocument.defaultView);
+      EventUtils.synthesizeMouse(element, 1, 1, {}, ownerDoc.defaultView);
       this.sleep(0);
 
       // Scroll down until item is visible
       for (var i = s = element.selectedIndex; i <= element.itemCount + s; ++i) {
         var entry = element.getItemAtIndex((i + 1) % element.itemCount);
-        EventUtils.synthesizeKey("VK_DOWN", {}, element.ownerDocument.defaultView);
+        EventUtils.synthesizeKey("VK_DOWN", {}, ownerDoc.defaultView);
         if (entry.label == item.label) {
           break;
         }
         else if (entry.label == "") i += 1;
       }
-
-      EventUtils.synthesizeMouse(item, 1, 1, {}, item.ownerDocument.defaultView);
+      
+      EventUtils.synthesizeMouse(item, 1, 1, {}, ownerDoc.defaultView);
       this.sleep(0);
-
-   frame.events.pass({'function':'Controller.select()'});
-   return true;
+      
+      frame.events.pass({'function':'Controller.select()'});
+      return true;
     } catch (ex) {
       throw new Error('No item selected for element ' + el.getInfo());
       return false;
@@ -1125,36 +1149,57 @@ MozMillController.prototype.mouseMove = function (doc, start, dest) {
   return true;
 }
 
-//Drag one eleent to the top x,y coords of another specified element
-MozMillController.prototype.dragDropElemToElem = function (dstart, ddest) {
-  //Get the drag and dest
-  var drag = dstart.getNode();
-  var dest = ddest.getNode();
+// Drag an element to the specified offset on another element, firing mouse and drag events
+//  win must be the window both elements are in. Adapted from EventUtils' synthesizeDrop()
+MozMillController.prototype.dragToElement = function(src, dest, offsetX,
+    offsetY, aWindow, dropEffect, dragData) {
+  srcElement = src.getNode();
+  destElement = dest.getNode();
+  aWindow = aWindow || srcElement.ownerDocument.defaultView;
+  offsetX = offsetX || 20;
+  offsetY = offsetY || 20;
 
-  //if one of these elements couldn't be looked up
-  if (!drag){
-    throw new Error("could not find element " + drag.getInfo());
-    return false;
-  }
-  if (!dest){
-    throw new Error("could not find element " + dest.getInfo());
-    return false;
+  var dataTransfer;
+
+  var trapDrag = function(event) {
+    dataTransfer = event.dataTransfer;
+    if(!dragData)
+      return;
+
+    for (var i = 0; i < dragData.length; i++) {
+      var item = dragData[i];
+      for (var j = 0; j < item.length; j++) {
+        dataTransfer.mozSetDataAt(item[j].type, item[j].data, i);
+      }
+    }
+    dataTransfer.dropEffect = dropEffect || "move";
+    event.preventDefault();
+    event.stopPropagation();
   }
 
-  var dragCoords = null;
-  var destCoords = null;
+  aWindow.addEventListener("dragstart", trapDrag, true);
+  EventUtils.synthesizeMouse(srcElement, 2, 2, { type: "mousedown" }, aWindow);
+  EventUtils.synthesizeMouse(srcElement, 11, 11, { type: "mousemove" }, aWindow);
+  EventUtils.synthesizeMouse(srcElement, offsetX, offsetY, { type: "mousemove" }, aWindow);
+  aWindow.removeEventListener("dragstart", trapDrag, true);
 
-  dragCoords = drag.getBoundingClientRect();
-  destCoords = dest.getBoundingClientRect();
+  var event = aWindow.document.createEvent("DragEvents");
+  event.initDragEvent("dragenter", true, true, aWindow, 0, 0, 0, 0, 0, false, false, false, false, 0, null, dataTransfer);
+  destElement.dispatchEvent(event);
 
-  //Do the initial move to the drag element position
-  events.triggerMouseEvent(drag.ownerDocument.body, 'mousemove', true, dragCoords.left, dragCoords.top);
-  events.triggerMouseEvent(drag, 'mousedown', true, dragCoords.left, dragCoords.top); //do the mousedown
-  events.triggerMouseEvent(drag.ownerDocument.body, 'mousemove', true, destCoords.left, destCoords.top);
-  events.triggerMouseEvent(dest, 'mouseup', true, destCoords.left, destCoords.top);
-  events.triggerMouseEvent(dest, 'click', true, destCoords.left, destCoords.top);
-  frame.events.pass({'function':'Controller.dragDropElemToElem()'});
-  return true;
+  var event = aWindow.document.createEvent("DragEvents");
+  event.initDragEvent("dragover", true, true, aWindow, 0, 0, 0, 0, 0, false, false, false, false, 0, null, dataTransfer);
+  if (destElement.dispatchEvent(event)) {
+    EventUtils.synthesizeMouse(destElement, offsetX, offsetY, { type: "mouseup" }, aWindow);
+    return "none";
+  }
+
+  event = aWindow.document.createEvent("DragEvents");
+  event.initDragEvent("drop", true, true, aWindow, 0, 0, 0, 0, 0, false, false, false, false, 0, null, dataTransfer);
+  destElement.dispatchEvent(event);
+  EventUtils.synthesizeMouse(destElement, offsetX, offsetY, { type: "mouseup" }, aWindow);
+
+  return dataTransfer.dropEffect;
 }
 
 function preferencesAdditions(controller) {
