@@ -45,6 +45,8 @@ var strings = {}; Components.utils.import('resource://mozmill/stdlib/strings.js'
 var arrays = {};  Components.utils.import('resource://mozmill/stdlib/arrays.js', arrays);
 var withs = {};   Components.utils.import('resource://mozmill/stdlib/withs.js', withs);
 var utils = {};   Components.utils.import('resource://mozmill/modules/utils.js', utils);
+var securableModule = {};
+  Components.utils.import('resource://mozmill/stdlib/securable-module.js', securableModule);
 
 var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].
      getService(Components.interfaces.nsIConsoleService);
@@ -95,6 +97,20 @@ var loadFile = function(path, collector) {
   module.Cc = Components.classes;
   module.Ci = Components.interfaces;
   module.Cu = Components.utils;
+  module.require = function (mod) {
+    var loader = new securableModule.Loader({
+      rootPaths: [ios.newFileURI(file.parent).spec],
+      defaultPrincipal: "system",
+      globals : { mozmill: mozmill,
+                  elementslib: elementslib,
+                  persisted: persisted,
+                  Cc: Components.classes,
+                  Ci: Components.interfaces,
+                  Cu: Components.utils }
+    });
+    return loader.require(mod);
+  }
+  
   if (collector != undefined) {
     collector.current_file = file;
     collector.current_path = path;
@@ -102,6 +118,16 @@ var loadFile = function(path, collector) {
   try {
     loader.loadSubScript(uri, module);
   } catch(e) {
+    events.fail(e);
+    var obj = {
+      'filename':path,
+      'passed':false,
+      'failed':true,
+      'passes':0,
+      'fails' :1,
+      'name'  :'Unknown Test',
+    };
+    events.fireEvent('endTest', obj);
     Components.utils.reportError(e);
   }
   
@@ -315,8 +341,10 @@ Collector.prototype.startHttpd = function () {
   }
 }
 Collector.prototype.stopHttpd = function () {
-  this.httpd.stop()
-  this.httpd = null;
+  if (this.httpd) {
+    this.httpd.stop(function(){});  // Callback needed to pause execution until the server has been properly shutdown
+    this.httpd = null;
+  }
 }
 Collector.prototype.addHttpResource = function (directory, ns) {
   if (!this.httpd) {
@@ -415,7 +443,6 @@ Collector.prototype.initTestDirectory = function (directory) {
   recursiveModuleLoader(os.getFileForPath(directory));
 }
   
-  
 // Observer which gets notified when the application quits
 function AppQuitObserver() {
   this.register();
@@ -469,6 +496,7 @@ Runner.prototype.end = function () {
   } catch(e) {
     events.fireEvent('error', "persist serialization failed.");
   }
+  this.collector.stopHttpd();
   events.fireEvent('endRunner', true);
 }
 Runner.prototype.getDependencies = function (module) {
@@ -547,6 +575,7 @@ Runner.prototype._runTestModule = function (module) {
   for (var i in module) {
     attrs.push(i);
   }
+  
   events.setModule(module);
   module.__status__ = 'running';
   if (module.__setupModule__) { 
